/*
 *  PureBasic Header File
 *  (c) Fantaisie Software
 *
 * Note: this file is shared between MacOS and Linux
 */


#ifndef PB_GADGET_H
#define PB_GADGET_H

#include <Event/Event.h>
#include <Object/Object.h>
#include <System/System.h>
#include <Window/Gtk/Window.h>

// PB_IMAGETYPE is dependent on the GdkPixbuf use
#define M_ImageType       PB_IMAGETYPE *
#define M_GadgetType      GtkWidget *

#ifdef GTK_2
	#define M_FontType      PangoFontDescription *
#else
  #define M_FontType      GdkFont *	
#endif
  
#define M_OptionGroupType GSList *
#define M_CharType        char *

#define GetWidgetID(Widget)     (((integer)g_object_get_data(G_OBJECT(Widget), "pb_id"))-1)
#define SetWidgetID(Widget, ID) g_object_set_data(G_OBJECT(Widget), "pb_id", (gpointer)(ID+1))

// redefine all the function which need a thread safe behaviour
//
#undef PB_AddGadgetColumn
#undef PB_AddGadgetItem
#undef PB_AddGadgetItem2
#undef PB_AddGadgetItem3
#undef PB_GadgetToolTip
#undef PB_SetGadgetText
#undef PB_SetGadgetItemText
#undef PB_SetGadgetItemText2
#undef PB_GetGadgetText
#undef PB_GetGadgetItemText
#undef PB_GetGadgetItemText2

#define PB_AddGadgetColumn    M_ThreadUnicodeFunction(PB_AddGadgetColumn)
#define PB_AddGadgetItem      M_ThreadUnicodeFunction(PB_AddGadgetItem)
#define PB_AddGadgetItem2     M_ThreadUnicodeFunction(PB_AddGadgetItem2)
#define PB_AddGadgetItem3     M_ThreadUnicodeFunction(PB_AddGadgetItem3)
#define PB_GadgetToolTip      M_ThreadUnicodeFunction(PB_GadgetToolTip)
#define PB_SetGadgetText      M_ThreadUnicodeFunction(PB_SetGadgetText)
#define PB_SetGadgetItemText  M_ThreadUnicodeFunction(PB_SetGadgetItemText)
#define PB_SetGadgetItemText2 M_ThreadUnicodeFunction(PB_SetGadgetItemText2)
#define PB_GetGadgetItemText  M_ThreadUnicodeFunction(PB_GetGadgetItemText)
#define PB_GetGadgetItemText2 M_ThreadUnicodeFunction(PB_GetGadgetItemText2)
#define PB_GetGadgetText      M_ThreadUnicodeFunction(PB_GetGadgetText)

#define PB_ClearGadgetItems     M_ThreadFunction(PB_ClearGadgetItems)
#define PB_RemoveGadgetColumn   M_ThreadFunction(PB_RemoveGadgetColumn)
#define PB_RemoveGadgetItem     M_ThreadFunction(PB_RemoveGadgetItem)
#define PB_SetGadgetAttribute   M_ThreadFunction(PB_SetGadgetAttribute)
#define PB_SetGadgetColor       M_ThreadFunction(PB_SetGadgetColor)
#define PB_SetGadgetFont        M_ThreadFunction(PB_SetGadgetFont)
#define PB_SetGadgetItemData    M_ThreadFunction(PB_SetGadgetItemData)
#define PB_SetGadgetItemState   M_ThreadFunction(PB_SetGadgetItemState)
#define PB_SetGadgetState       M_ThreadFunction(PB_SetGadgetState)
#define PB_ResizeGadget         M_ThreadFunction(PB_ResizeGadget)
#define PB_SetGadgetItemAttribute   M_ThreadFunction(PB_SetGadgetItemAttribute)
#define PB_SetGadgetItemAttribute2  M_ThreadFunction(PB_SetGadgetItemAttribute2)

/* Gadget Coloring options:
 */
#define PB_Gadget_FrontColor      1
#define PB_Gadget_BackColor       2
#define PB_Gadget_LineColor       3
#define PB_Gadget_TitleFrontColor 4
#define PB_Gadget_TitleBackColor  5
#define PB_Gadget_GreyTextColor   6

enum {  // gadget types
  PB_GadgetType_Unknown = 0,

  PB_GadgetType_Button,
  PB_GadgetType_String,
  PB_GadgetType_Text,
  PB_GadgetType_CheckBox,
  PB_GadgetType_Option,
  PB_GadgetType_ListView,
  PB_GadgetType_Frame,
  PB_GadgetType_ComboBox,
  PB_GadgetType_Image,
  PB_GadgetType_HyperLink,
  PB_GadgetType_Container,
  PB_GadgetType_ListIcon,
  PB_GadgetType_IPAddress,
  PB_GadgetType_ProgressBar,
  PB_GadgetType_ScrollBar,
  PB_GadgetType_ScrollArea,
  PB_GadgetType_TrackBar,
  PB_GadgetType_Web,
  PB_GadgetType_ButtonImage,
  PB_GadgetType_Calendar,
  PB_GadgetType_Date,
  PB_GadgetType_Editor,
  PB_GadgetType_ExplorerList,
  PB_GadgetType_ExplorerTree,
  PB_GadgetType_ExplorerCombo,
  PB_GadgetType_Spin,
  PB_GadgetType_Tree,
  PB_GadgetType_Panel,
  PB_GadgetType_Splitter,
  PB_GadgetType_MDI,
  PB_GadgetType_Scintilla,
  PB_GadgetType_Shortcut,
  PB_GadgetType_Canvas,
	PB_GadgetType_OpenGL,

  PB_GadgetType_LastEnum // to easily know the size of the enumeration
};

struct PB_GadgetStructure;

typedef int  (*AddGadgetColumnProc)        (struct PB_GadgetStructure *Gadget, int Position, const M_CharType Title, int Width);
typedef integer (*AddGadgetItem2Proc)      (struct PB_GadgetStructure *Gadget, int Position, const M_CharType Text, M_ImageType Picture);
typedef integer (*AddGadgetItem3Proc)      (struct PB_GadgetStructure *Gadget, int Position, const M_CharType Text, M_ImageType Picture, int Flags);
typedef void (*ClearGadgetItemsProc)       (struct PB_GadgetStructure *Gadget);
typedef int  (*CountGadgetItemsProc)       (struct PB_GadgetStructure *Gadget);
typedef integer (*GetGadgetAttributeProc)  (struct PB_GadgetStructure *Gadget, int Attribute);
typedef void*(*GetGadgetFontProc)          (struct PB_GadgetStructure *Gadget);
typedef integer (*GetGadgetItemAttribute2Proc)(struct PB_GadgetStructure *Gadget, int ItemIndex, int Attribute, int Column);
typedef int  (*GetGadgetItemStateProc)     (struct PB_GadgetStructure *Gadget, int Item);
typedef void (*GetGadgetItemTextProc)      (struct PB_GadgetStructure *Gadget, int Item, int Column, int PreviousPosition);
typedef integer (*GetGadgetStateProc)      (struct PB_GadgetStructure *Gadget);
typedef void (*GetGadgetTextProc)          (struct PB_GadgetStructure *Gadget, int PreviousPosition);
typedef void (*OpenGadgetList2Proc)        (struct PB_GadgetStructure *Gadget, int ItemID);
typedef void (*RemoveGadgetColumnProc)     (struct PB_GadgetStructure *Gadget, int Position);
typedef void (*RemoveGadgetItemProc)       (struct PB_GadgetStructure *Gadget, int Position);
typedef void (*ResizeGadgetProc)           (struct PB_GadgetStructure *Gadget, int x, int y, int Width, int Height);
typedef void (*ActivateGadgetProc)         (struct PB_GadgetStructure *Gadget);
typedef void (*SetGadgetAttributeProc)     (struct PB_GadgetStructure *Gadget, int Attribute, integer Value);
typedef void (*SetGadgetFontProc)          (struct PB_GadgetStructure *Gadget, M_FontType Font);
typedef void (*SetGadgetItemAttribute2Proc)(struct PB_GadgetStructure *Gadget, int ItemIndex, int Attribute, integer Value, int Column);
typedef void (*SetGadgetItemStateProc)     (struct PB_GadgetStructure *Gadget, int Item, int State);
typedef void (*SetGadgetItemTextProc)      (struct PB_GadgetStructure *Gadget, int Item, const M_CharType Text, int Column);
typedef void (*SetGadgetStateProc)         (struct PB_GadgetStructure *Gadget, integer State);
typedef void (*SetGadgetTextProc)          (struct PB_GadgetStructure *Gadget, const M_CharType NewText);
typedef void (*FreeGadgetProc)             (struct PB_GadgetStructure *Gadget);
typedef void (*SetGadgetColorProc)         (struct PB_GadgetStructure *Gadget, int ColorType, int Color);
typedef int  (*GetGadgetColorProc)         (struct PB_GadgetStructure *Gadget, int ColorType);
typedef void (*SetGadgetItemColor2Proc)    (struct PB_GadgetStructure *Gadget, int Item, int ColorType, int Color, int Column);
typedef int  (*GetGadgetItemColor2Proc)    (struct PB_GadgetStructure *Gadget, int Item, int ColorType, int Column);
typedef void (*SetGadgetItemDataProc)      (struct PB_GadgetStructure *Gadget, int Item, void *Value);
typedef void*(*GetGadgetItemDataProc)      (struct PB_GadgetStructure *Gadget, int Item);
typedef void (*SetGadgetItemImageProc)     (struct PB_GadgetStructure *Gadget, int Item, M_ImageType Image);


typedef struct StructureVT
{
  int     SizeOf;
  int     GadgetType;

  ActivateGadgetProc          ActivateGadget;
  FreeGadgetProc              FreeGadget;
  GetGadgetStateProc          GetGadgetState;
  SetGadgetStateProc          SetGadgetState;
  GetGadgetTextProc           GetGadgetText;
  SetGadgetTextProc           SetGadgetText;
  AddGadgetItem2Proc          AddGadgetItem2;
  AddGadgetItem3Proc          AddGadgetItem3;
  RemoveGadgetItemProc        RemoveGadgetItem;
  ClearGadgetItemsProc     ClearGadgetItems;
  ResizeGadgetProc            ResizeGadget;
  CountGadgetItemsProc        CountGadgetItems;
  GetGadgetItemStateProc      GetGadgetItemState;
  SetGadgetItemStateProc      SetGadgetItemState;
  GetGadgetItemTextProc       GetGadgetItemText;
  SetGadgetItemTextProc       SetGadgetItemText;
  SetGadgetFontProc           SetGadgetFont;
  OpenGadgetList2Proc         OpenGadgetList2;
  AddGadgetColumnProc         AddGadgetColumn;
  GetGadgetAttributeProc      GetGadgetAttribute;
  SetGadgetAttributeProc      SetGadgetAttribute;
  GetGadgetItemAttribute2Proc GetGadgetItemAttribute2;
  SetGadgetItemAttribute2Proc SetGadgetItemAttribute2;
  RemoveGadgetColumnProc      RemoveGadgetColumn;
  SetGadgetColorProc          SetGadgetColor;
  GetGadgetColorProc          GetGadgetColor;
  SetGadgetItemColor2Proc     SetGadgetItemColor2;
  GetGadgetItemColor2Proc     GetGadgetItemColor2;
  SetGadgetItemDataProc       SetGadgetItemData;
  GetGadgetItemDataProc       GetGadgetItemData;
  GetGadgetFontProc           GetGadgetFont;
  SetGadgetItemImageProc      SetGadgetItemImage;
} PB_GadgetVT;


typedef struct PB_GadgetStructure
{
  M_GadgetType Gadget;
  M_GadgetType Container;  // Container gadget, as a listIcon can be in a ScrolledWindow and when we use the splittergadget, we get tricked..
  PB_GadgetVT *VT;
  integer RootWindowID; // In the 'destroy' handler, dynamic way to get the root window id fails (with PB_Gadget_GetRootWindowID()), we need a cache 
  integer UserData;
  integer Data[6];
} PB_Gadget;


typedef struct
{
  M_GadgetType Current; // must be the first for a small hack in the IDE sources

  int          StackIndex;
  M_GadgetType Stack[100];

  M_FontType   DefaultFont;

  M_OptionGroupType OptionGroup;

  GtkWidget *OptionGroupButton;
  GtkWidget *Window; // current GadgetList toplevel (does not have to be a GtkWindow!)
  //  int        RawButtonEvent;

} PB_GadgetGlobals;


#define InitializeVT(VT, Type) { memset(&VT, 0, sizeof(PB_GadgetVT)); VT.SizeOf = sizeof(PB_GadgetVT); VT.GadgetType = Type; }

#define PushGadgetStack(Widget) { Globals->Stack[Globals->StackIndex] = Globals->Current; \
                                  Globals->StackIndex++; \
                                  Globals->Current = Widget; }

#define PushGadgetStackNoCurrent() { Globals->Stack[Globals->StackIndex] = Globals->Current; \
                                     Globals->StackIndex++; }

// For compatibility with Windows Debug file
#define PB_Gadget_CurrentObject    PB_Gadget_Current
#define PB_Gadget_PanelStackOffset PB_Gadget_Stack


// Common flags for all 'Explorer' gadgets
//
#define PB_Explorer_NoMyDocuments       (1 << 0)
#define PB_Explorer_HiddenFiles         (1 << 18)

// Common flags for ExplorerListGadget() and ExplorerTreeGadget()
//
#define PB_Explorer_NoFiles             (1 << 1)
#define PB_Explorer_NoDriveRequester    (1 << 2)
#define PB_Explorer_AutoSort            (1 << 3)
#define PB_Explorer_BorderLess          (1 << 4)
#define PB_Explorer_AlwaysShowSelection (1 << 5)

// ExplorerList only Flags:
#define PB_Explorer_NoParentFolder      (1 << 6)
#define PB_Explorer_NoFolders           (1 << 7)
#define PB_Explorer_NoDirectoryChange   (1 << 8)
#define PB_Explorer_NoSort              (1 << 9)
#define PB_Explorer_MultiSelect         (1 << 10)
#define PB_Explorer_GridLines           (1 << 11)
#define PB_Explorer_HeaderDragDrop      (1 << 12)
#define PB_Explorer_FullRowSelect       (1 << 13)

// ExplorerTree only Flags:
#define PB_Explorer_NoLines             (1 << 14)
#define PB_Explorer_NoButtons           (1 << 15)

// ExplorerCombo only Flags:
#define PB_Explorer_DrivesOnly          (1 << 16)
#define PB_Explorer_Editable            (1 << 17)

// Explorerlist standart columns:
#define PB_Explorer_Name                "PB_Explorer_Column_Name"
#define PB_Explorer_Size                "PB_Explorer_Column_Size"
#define PB_Explorer_Type                "PB_Explorer_Column_Type"
#define PB_Explorer_Attributes          "PB_Explorer_Column_Attributes"
#define PB_Explorer_Created             "PB_Explorer_Column_Created"
#define PB_Explorer_Modified            "PB_Explorer_Column_Modified"
#define PB_Explorer_Accessed            "PB_Explorer_Column_Accessed"

#define PB_Explorer_ColumnWidth         (1 << 19)

// Common return values for Explorer gadgets
//
#define PB_Explorer_File       1
#define PB_Explorer_Directory  2
#define PB_Explorer_Selected   4

#define PB_Editor_ReadOnly 1
#define PB_Editor_WordWrap 2

// Attributes
//
#define PB_Spin_Minimum 1
#define PB_Spin_Max 2

// Flags
//
#define PB_Spin_ReadOnly 4
#define PB_Spin_Numeric 8

#define PB_Panel_ItemWidth  1
#define PB_Panel_ItemHeight 2
#define PB_Panel_TabHeight  3

#define PB_ListView_MultiSelect 1
#define PB_ListView_ClickSelect 2

#define PB_ListIcon_MultiSelect    1
#define PB_ListIcon_CheckBoxes     2
#define PB_ListIcon_GridLines      4
#define PB_ListIcon_ThreeState     8
#define PB_ListIcon_HeaderDragDrop 16
#define PB_ListIcon_FullRowSelect  0 // Not used
#define PB_ListIcon_AlwaysShowSelection 0 // Not used

// GetGadgetAttribute()
#define PB_ListIcon_ColumnCount    3

// GetGadgetItemAttribute()
#define PB_ListIcon_ColumnWidth    1

#define PB_ListIcon_Selected       1
#define PB_ListIcon_Checked        2
#define PB_ListIcon_Inbetween      4

#define PB_Image_Border            (1 << 0)
#define PB_Image_Raised						 (1 << 1)

#define PB_Button_Default         0x8
#define PB_Button_Left            0x2
#define PB_Button_MultiLine       0x10
#define PB_Button_Right           0x1
#define PB_Button_Toggle          0x4

#define PB_Button_Image           1
#define PB_Button_PressedImage    2

#define PB_Calendar_Borderless    1 // unsupported, check the residents if used

#define PB_Calendar_Minimum       1
#define PB_Calendar_Maximum       2

#define PB_CheckBox_Right         1
#define PB_CheckBox_Center        2
#define PB_CheckBox_ThreeState    4

#define PB_ComboBox_Editable      1
#define PB_ComboBox_LowerCase     2
#define PB_ComboBox_UpperCase     4
#define PB_ComboBox_Image         8

#define PB_Date_Calendar 0 // default
#define PB_Date_UpDown   1
#define PB_Date_CheckBox 2

#define PB_ScrollArea_InnerWidth 1
#define PB_ScrollArea_InnerHeight 2
#define PB_ScrollArea_X 3
#define PB_ScrollArea_Y 4
#define PB_ScrollArea_ScrollStep 5

#define PB_ScrollArea_Flat  1
#define PB_ScrollArea_Raised  2
#define PB_ScrollArea_Single  4
#define PB_ScrollArea_BorderLess  8
#define PB_ScrollArea_Center  16

#define PB_Splitter_Vertical 1
#define PB_Splitter_Separator 2
#define PB_Splitter_FirstFixed 4
#define PB_Splitter_SecondFixed 8

#define PB_Splitter_FirstMinimumSize 1
#define PB_Splitter_SecondMinimumSize 2
#define PB_Splitter_FirstGadget 3
#define PB_Splitter_SecondGadget 4

#define PB_Tree_Selected  1
#define PB_Tree_Expanded  2
#define PB_Tree_Checked   4
#define PB_Tree_Collapsed 8
#define PB_Tree_Inbetween 16

#define PB_Tree_Sublevel  1

#define PB_Tree_CheckBoxes 4
#define PB_Tree_AlwaysShowSelection 0
#define PB_Tree_NoButtons  2
#define PB_Tree_NoLines    1
#define PB_Tree_ThreeState 8

#define PB_Editor_ReadOnly  1

#define PB_Container_BorderLess  0
#define PB_Container_Flat        1
#define PB_Container_Raised      2
#define PB_Container_Single      4
#define PB_Container_Double      8

#define PB_Frame_Double    1
#define PB_Frame_Single    2
#define PB_Frame_Flat      3

#define PB_Text_Right 1
#define PB_Text_Center 2
#define PB_Text_Border 4

#define PB_String_Password 1
#define PB_String_ReadOnly 2
#define PB_String_UpperCase 4
#define PB_String_LowerCase 8
#define PB_String_Numeric  16
#define PB_String_BorderLess 32
#define PB_String_AutoComplete   64
#define PB_String_AutoInsert     128

#define PB_ProgressBar_Smooth  0
#define PB_ProgressBar_Vertical 1

#define PB_ProgressBar_Minimum  1
#define PB_ProgressBar_Maximum  2

#define PB_ScrollBar_Vertical  1

#define PB_ScrollBar_Minimum     1
#define PB_ScrollBar_Maximum     2
#define PB_ScrollBar_PageLength  3

#define PB_TrackBar_Minimum  1
#define PB_TrackBar_Maximum  2

#define PB_TrackBar_Ticks    1
#define PB_TrackBar_Vertical 2


#define PB_HyperLink_Underline 1

// for webgadget
// SetGadgetState() constants
//
#define PB_Web_Back     1
#define PB_Web_Forward  2
#define PB_Web_Stop     3
#define PB_Web_Refresh  4

// for Get/SetGadgetItemText()
//
#define PB_Web_HtmlCode      1
#define PB_Web_PageTitle     2
#define PB_Web_StatusMessage 3
#define PB_Web_SelectedText  4

#define PB_EventType_Up                4 // Used by SpinGadget()
#define PB_EventType_Down              5 // Used by SpinGadget()

// for Get/SetGadgetAttribute()
//
#define PB_Web_BlockPopups        1
#define PB_Web_BlockPopupMenu     2
#define PB_Web_NavigationCallback 3
#define PB_Web_Progress           4
#define PB_Web_ProgressMax        5
#define PB_Web_Busy               6
#define PB_Web_ScrollX            7
#define PB_Web_ScrollY            8


extern PB_Object *PB_Gadget_Objects;
extern integer    PB_Gadget_Globals;
extern GtkStyle  *PB_Gadget_DefaultStyle;

#define PB_Gadget_GetOrAllocateIDAndFree PB_Gadget_GetOrAllocateID
#define PB_Gadget_GetOrAllocateID(ID)   ((PB_Gadget *)PB_Object_GetOrAllocateID(PB_Gadget_Objects, ID))
#define PB_Gadget_GetObject(ID)         ((PB_Gadget *)PB_Object_GetObject(PB_Gadget_Objects, ID))
#define PB_Gadget_FreeID(ID)                          PB_Object_FreeID(PB_Gadget_Objects, ID)

M_PBFUNCTION(void)         PB_FreeGadget(integer GadgetID);
M_PBFUNCTION(PB_Gadget *)  PB_IsGadget(integer GadgetID);
M_PBFUNCTION(void) PB_ClearGadgetItems(integer GadgetID);
M_PBFUNCTION(void) PB_SetGadgetFont(integer GadgetID, M_FontType Font);
M_PBFUNCTION(void) PB_ClearGadgetItems(integer GadgetID);
M_PBFUNCTION(integer) PB_GadgetX(integer GadgetID);
M_PBFUNCTION(integer) PB_GadgetX2(integer GadgetID, int Flags);
M_PBFUNCTION(integer) PB_GadgetY(integer GadgetID);
M_PBFUNCTION(integer) PB_GadgetY2(integer GadgetID, int Flags);
M_PBFUNCTION(integer) PB_GadgetWidth(integer GadgetID);
M_PBFUNCTION(integer) PB_GadgetHeight(integer GadgetID);
M_PBFUNCTION(integer) PB_CountGadgetItems(integer GadgetID);

#include <Gadget/GadgetCommon.h>

void PB_Gadget_FinishCreation(PB_Gadget *Gadget, integer GadgetID);
void PB_Gadget_AddWidget(PB_GadgetGlobals *Globals, GtkWidget *Widget, int x, int y);

void PB_Gadget_SendEvent        (void *Widget, integer EventType);
void PB_Gadget_FlushEvents      ();
void PB_Gadget_EventHandler     (GtkButton* button, integer ButtonID);
void PB_Gadget_UpperCaseHandler (GtkEditable *editable, const gchar *text, gint length, gint *position, gpointer data);
void PB_Gadget_LowerCaseHandler (GtkEditable *editable, const gchar *text, gint length, gint *position, gpointer data);
void PB_Gadget_NumericHandler   (GtkEditable *editable, const gchar *text, gint length, gint *position, gpointer data);
void PB_Gadget_ResizeHandler    (GtkWidget *Widget, GdkRectangle *Allocation, PB_Gadget *Gadget);
void PB_Gadget_TextHandler      (GtkEditable *editable, const gchar *text, gint length, gint *position, gpointer data);
void*PB_Gadget_GetFont          (PB_Gadget *Gadget);
void*PB_Gadget_GetWidgetFont    (GtkWidget *Widget);
void PB_Gadget_SetFont          (PB_Gadget *Gadget, M_FontType Font);
void PB_Gadget_SetWidgetFont    (GtkWidget *Widget, M_FontType Font);
void PB_Gadget_SetColor(PB_Gadget *Gadget, int ColorType, int Color);
void PB_Gadget_SetColorForWidget(PB_Gadget *Gadget, int ColorType, int Color, GtkWidget *Widget);
int  PB_Gadget_GetColor(PB_Gadget *Gadget, int ColorType);
integer    PB_Gadget_GetRootWindowID(GtkWidget *Widget);
GtkWidget *PB_Gadget_GetRootWindow(GtkWidget *Widget);
void PB_Gadget_SetDragStartHandler(integer GadgetID, GtkWidget *Widget);
void PB_Gadget_ResizeWidget(GtkWidget *Container, int x, int y, int Width, int Height);
gboolean PB_Gadget_MouseHandler(GtkWidget *widget, GdkEventButton *event, integer GadgetID);
gboolean PB_Gadget_FocusHandler    (GtkWidget *widget, GdkEventFocus *event, integer ButtonID);
gboolean PB_Gadget_LostFocusHandler(GtkWidget *widget, GdkEventFocus *event, integer ButtonID);
void     PB_Gadget_ChangedHandler  (GtkEditable *editable, integer ButtonID);
void     PB_Gadget_ReturnAcceleratorHandler(GtkWidget *Gadget, integer GadgetID);

void pb_widget_get_rect_in_screen(GtkWidget *widget, GdkRectangle *r);

void PB_Event_AddRawButtonControl(GtkWidget *Widget, int RawButtonEvent);

#ifdef GTK_3
  void PB_Gadget_FreeCssProviders(PB_Gadget *Gadget);

  #define M_GtkPanelSeparatorThickness 5
#else
  #define M_GtkPanelSeparatorThickness 5

  // Nothing to do
  #define PB_Gadget_FreeCssProviders(a)
#endif


#ifdef GTK_2

  // GtkTreeView shared functions...

  #define PB_TreeView_SelectionHandler    M_UnicodeFunction(PB_TreeView_SelectionHandler)
  #define PB_TreeView_ExtendListStore     M_UnicodeFunction(PB_TreeView_ExtendListStore)
  #define PB_TreeView_SelectItem          M_UnicodeFunction(PB_TreeView_SelectItem)
  #define PB_TreeView_ClearGadgetItems M_UnicodeFunction(PB_TreeView_ClearGadgetItems)
  #define PB_TreeView_CountGadgetItems    M_UnicodeFunction(PB_TreeView_CountGadgetItems)
  #define PB_TreeView_RemoveGadgetItem    M_UnicodeFunction(PB_TreeView_RemoveGadgetItem)
  #define PB_TreeView_SetGadgetState      M_UnicodeFunction(PB_TreeView_SetGadgetState)
  #define PB_TreeView_GetGadgetState      M_UnicodeFunction(PB_TreeView_GetGadgetState)
  #define PB_TreeView_SetGadgetItemData   M_UnicodeFunction(PB_TreeView_SetGadgetItemData)
  #define PB_TreeView_GetGadgetItemData   M_UnicodeFunction(PB_TreeView_GetGadgetItemData)
  #define PB_TreeView_SetGadgetItemText   M_UnicodeFunction(PB_TreeView_SetGadgetItemText)
  #define PB_TreeView_GetGadgetItemText   M_UnicodeFunction(PB_TreeView_GetGadgetItemText)
  #define PB_TreeView_SetColumnFont       M_UnicodeFunction(PB_TreeView_SetColumnFont)
  #define PB_TreeView_SetGadgetFont       M_UnicodeFunction(PB_TreeView_SetGadgetFont)

  void     PB_TreeView_SelectionHandler(GtkTreeSelection *treeselection, GtkWidget *widget);
  int      PB_TreeView_ExtendListStore(PB_Gadget *Gadget, int nbcolumns, GType *Columns);
  void     PB_TreeView_SelectItem(PB_Gadget *Gadget, GtkTreeIter *Item);

  void PB_TreeView_ClearGadgetItems(PB_Gadget *Gadget);
  int  PB_TreeView_CountGadgetItems(PB_Gadget *Gadget);
  void PB_TreeView_RemoveGadgetItem(PB_Gadget *Gadget, int Index);
  void PB_TreeView_SetGadgetState(PB_Gadget *Gadget, integer Index);
  integer PB_TreeView_GetGadgetState(PB_Gadget *Gadget);
  void PB_TreeView_SetGadgetItemData(PB_Gadget *Gadget, int Position, void *Data);
  void *PB_TreeView_GetGadgetItemData(PB_Gadget *Gadget, int Position);
  void PB_TreeView_SetGadgetItemText(PB_Gadget *Gadget, int RowIndex, const char *Text, int Column);
  void PB_TreeView_GetGadgetItemText(PB_Gadget *Gadget, int RowIndex, int Column, int PreviousPosition);
  void PB_TreeView_SetColumnFont(PB_Gadget *Gadget, GtkTreeViewColumn *Column);
  void PB_TreeView_SetGadgetFont(PB_Gadget *Gadget, M_FontType Font);

  // GtkTreeView info structure
  typedef struct
  {
    GtkListStore *Store;
    GtkTreeSelection *Selection;

    int DataColumn;     // column index of the column for user data of type G_TYPE_POINTER

    int NbTextColumns;  // number of text columns in the gadget
    GList *TextColumns; // list of text column indexes.

    /* Note: GList is used here instead of the PB SimpleList, because we need access to the nth element
     * which the SimpleList functions do not provide
     */

  } PB_TreeView_Data;

#else

	// GTK1 only functions
  void PB_Gadget_LoadCheckPixmaps(GdkWindow *Window);
  
  void PB_Gadget_GetListItemText  (GtkList *Gadget, int Position, int PreviousPosition);
  void PB_Gadget_SetListItemText  (GtkList *Gadget, int Position, const char *Text);
  void*PB_Gadget_GetListItemData  (GtkList *Gadget, int Position);
  void PB_Gadget_SetListItemData  (GtkList *Gadget, int Position, void *Value);

  extern GdkPixmap *PB_Gadget_UnCheckedPixmap;
  extern GdkPixmap *PB_Gadget_UnCheckedPixmapMask;
  extern GdkPixmap *PB_Gadget_CheckedPixmap;
  extern GdkPixmap *PB_Gadget_CheckedPixmapMask;

#endif

#endif
